#include "StdAfx.h"
#include "FileRecord.h"
#include <commdlg.h>
#include <shlwapi.h>
#include <shlobj.h>

#pragma comment(lib,"shlwapi.lib")

CFileRecord::CFileRecord(DWORD dwWriteBlockSize)
{
	m_pBuf=new CBuffer(max(1<<20,dwWriteBlockSize*2));
	ZeroMemory(m_pDir,MAX_PATH);
	ZeroMemory(m_pFile,MAX_PATH);
	m_hFile=INVALID_HANDLE_VALUE;
	m_dwBlockWriteSize=dwWriteBlockSize;
	m_dwRecordStartTick=0;
	m_ui64WriteSizeTotal=0;
}

CFileRecord::~CFileRecord(void)
{
	Close();
	m_pBuf->Clear();
	delete m_pBuf;
}

BOOL CFileRecord::GetRecordFileName(HWND hParentWnd,char *lpszPath,int nMaxPath)
{
	BOOL bRet;
	OPENFILENAMEA stOFN;
	ZeroMemory(&stOFN,sizeof(OPENFILENAME));
	stOFN.lStructSize=sizeof(OPENFILENAME);
	stOFN.hwndOwner=hParentWnd;
	stOFN.lpstrFile=lpszPath;
	stOFN.nMaxFile=nMaxPath;
	stOFN.lpstrFilter="TS Stream\0*.ts\0\0";
	stOFN.lpstrDefExt="ts";
	stOFN.Flags=OFN_EXTENSIONDIFFERENT|OFN_OVERWRITEPROMPT|OFN_NOCHANGEDIR;

	bRet=GetSaveFileNameA(&stOFN);
	return bRet;
}

BOOL CFileRecord::GetPathName(HWND hParentWnd,char *lpszPath,int nMaxPath)
{
	BROWSEINFO stBI;
	ZeroMemory(&stBI,sizeof(stBI));
	stBI.hwndOwner=hParentWnd;
	stBI.ulFlags = BIF_RETURNONLYFSDIRS;

	LPITEMIDLIST pList=SHBrowseForFolder(&stBI);
	if(pList)
	{
		SHGetPathFromIDListA(pList,lpszPath);
		CoTaskMemFree(pList);
		return TRUE;
	}
	return FALSE;
}

int CFileRecord::CreateDirs(char* dir,int depth)
{
	char *p_tmp=dir;

	DWORD attr=GetFileAttributesA(p_tmp);
	if(attr==INVALID_FILE_ATTRIBUTES)
	{
		char c,*p=strrchr(p_tmp,'\\');		
		if(p)
		{
			int ret;
			c=*p;
			*p='\0';
			if(strrchr(p_tmp,'\\'))
			{
				ret=CreateDirs(p_tmp,depth+1);
			} else {
				ret=1;
			}
			*p=c;
			if(ret==1)
			{
				if(CreateDirectoryA(p_tmp,NULL))
				{
					return 1;
				} else {
					return 0;
				}
			}

		}
		_asm {nop};
	} else {
		if(attr&FILE_ATTRIBUTE_DIRECTORY)
		{
			return 1;
		} else {
			return 0;
		}
	}
}

void CFileRecord::CheckExistAndRename(char *lpszPath)
{
	if(GetFileAttributesA(lpszPath)==INVALID_FILE_ATTRIBUTES) return;
	// t@Cłɑ݂ꍇ
	char szDir[MAX_PATH];
	char szFile[MAX_PATH];
	char szExt[MAX_PATH];
	char szPath[MAX_PATH];
	char szTmp[MAX_PATH];

	lstrcpyA(szDir,lpszPath);
	PathRemoveFileSpecA(szDir);
	lstrcpyA(szFile,PathFindFileNameA(lpszPath));
	lstrcpyA(szExt,PathFindExtensionA(szFile));
	PathRemoveExtensionA(szFile);

	// Aԃt@C쐬
	for(int i=1;;i++)
	{
		wsprintfA(szTmp,"%s_%d",szFile,i);
		lstrcpyA(szPath,szDir);
		PathAppendA(szPath,szTmp);
		PathAddExtensionA(szPath,szExt);
		if(GetFileAttributesA(szPath)==INVALID_FILE_ATTRIBUTES)
		{
			lstrcpyA(lpszPath,szPath);
			return;
		}
	}
}

BOOL CFileRecord::Open(char *lpszPath)
{
	if(IsOpen()) Close();
    lstrcpynA(m_pDir,lpszPath,MAX_PATH);
	lstrcpynA(m_pFile,lpszPath,MAX_PATH);
	PathRemoveFileSpecA(m_pDir);
	PathFindFileNameA(m_pFile);
	
	// FILE_ATTRIBUTE_NORMAL|FILE_FLAG_WRITE_THROUGH|FILE_FLAG_SEQUENTIAL_SCAN
	m_hFile=CreateFileA(lpszPath,GENERIC_WRITE,FILE_SHARE_READ,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(m_hFile==INVALID_HANDLE_VALUE) return FALSE;

	m_dwRecordStartTick=GetTickCount();

	return TRUE;;
}

void CFileRecord::Close()
{
	if(m_hFile!=INVALID_HANDLE_VALUE)
	{
		if(m_dwBlockWriteSize==0) _WriteBlock(0,NULL);
		CloseHandle(m_hFile);
		ZeroMemory(m_pDir,MAX_PATH);
		ZeroMemory(m_pFile,MAX_PATH);
		m_hFile=INVALID_HANDLE_VALUE;
	}
}

BOOL CFileRecord::IsOpen()
{
	return m_hFile!=INVALID_HANDLE_VALUE;
}

DWORD CFileRecord::GetOpenTick()
{
	if(!IsOpen()) return 0;
	return GetTickCount()-m_dwRecordStartTick;
}

// JĂt@CƏ߂̂͂ǂ񂾂
ULARGE_INTEGER CFileRecord::GetFreeSize()
{
	static ULARGE_INTEGER ui64Zero={0,0};
	ULARGE_INTEGER ui64FreeSpace,ui64TotalSize;
	if(!IsOpen()) return ui64Zero;

	if(GetDiskFreeSpaceExA(m_pDir,&ui64FreeSpace,&ui64TotalSize,NULL))
	{
		return ui64FreeSpace;
	} else {
		return ui64Zero;
	}
}

BOOL CFileRecord::Write(void *lpData,DWORD dwDataSize,DWORD *lpdwWritten)
{
	BOOL bRet=FALSE;
	if(!IsOpen()) return FALSE;
	if(m_dwBlockWriteSize==0)
	{		
		DWORD dwWritten;
		if(!lpdwWritten) lpdwWritten=&dwWritten;
		bRet=WriteFile(m_hFile,lpData,dwDataSize,lpdwWritten,NULL);
		if(bRet)
		{
			m_ui64WriteSizeTotal+=*lpdwWritten;
		}
	} else {
		DWORD dwSize;
		dwSize=m_pBuf->Put((BYTE*)lpData,dwDataSize);
		if(dwSize==dwDataSize)
		{
			if(m_pBuf->GetDataSize()>m_dwBlockWriteSize)
			{
				return _WriteBlock(m_dwBlockWriteSize,lpdwWritten);
			}
			if(lpdwWritten) *lpdwWritten=0;
			bRet=TRUE;
		} else {
			bRet=FALSE;
		}		
	}
	return bRet;
}

BOOL CFileRecord::_WriteBlock(DWORD dwWriteSize,DWORD *lpdwWritten)
{
	if(!IsOpen()) return FALSE;

	BOOL bRet=FALSE;
	DWORD dwSize,dwWritten;
	BYTE *pBuf=new BYTE[m_dwBlockWriteSize];

	if(m_dwBlockWriteSize>0)
	{
		dwSize=m_pBuf->Get(pBuf,dwWriteSize);
	} else {
		dwSize=m_pBuf->Get(pBuf,m_dwBlockWriteSize);
	}
	if(!lpdwWritten) lpdwWritten=&dwWritten;
	bRet=WriteFile(m_hFile,pBuf,dwSize,lpdwWritten,NULL);
	if(bRet)
	{
		m_ui64WriteSizeTotal+=*lpdwWritten;
	}

	delete []pBuf;
	return bRet;
}