#pragma once

#define FRIIO_SDK_VERSION (0x00010400)
#define FRIIO_SDK_MINOR(x) (x & 0xFF)
#define FRIIO_SDK_MAJOR(x) ((x >> 8) & 0xFF)
#define FRIIO_SDK_RELEASE(x) ((x >> 16) & 0xFF)

// Send to Player Main Window
#define WM_STARTREC	(WM_USER + 3000)	// start recording wParam = MAKEWPARAM(RecordTimeInMinutes, DelayBeforeRecordInSeconds)
#define WM_STOPREC	(WM_USER + 3001)	// stops recording
#define WM_CHANGECHAN	(WM_USER + 3002)	// wParam = channel idx (13-62 terrestrial, > 0xffff satellite)
#define WM_ISRECEIVING	(WM_USER + 3003)	// is data currently streaming (high CN, some bitrate)
#define WM_UPDATECHAN	(WM_USER + 3004)
#define WM_SCREENSHOT	(WM_USER + 3005)	// unused, ignored
#define WM_SETAUDIO	(WM_USER + 3006)	// set audio mode wParam = AUDIO_BOTH / AUDIO_PRIMARY / AUDIO_SECONDARY, lParam = AUDIO_ES_INDEX (-1 for not-set)
#define WM_GETAUDIO	(WM_USER + 3007)	// get audio mode, returns MAKELONG(AUDIO_BOTH | AUDIO_PRIMARY | AUDIO_SECONDARY, AUDIO_ES_INDEX)
#define WM_SETASPECT	(WM_USER + 3008)	// set aspect ratio handling lParam = 1 to block aspect ratio changes from EIT, 0 to unblock
#define WM_GETASPECT	(WM_USER + 3009)	// return aspect ratio, MAKEWPARAM(AspectX, AspectY)

#define WM_PROGRAMS_UPDATED (WM_USER + 4001)	// call if program/service count changes

// for dual language broadcast
enum {
    AUDIO_BOTH = 0,
    AUDIO_PRIMARY = 1,
    AUDIO_SECONDARY = 2
};

// for B-CAS card status
enum {
    DTV_CARD_OK,
    DTV_CARD_NOREADER,
    DTV_CARD_NOCARD,
    DTV_CARD_REINSERTCARD,
    DTV_CARD_ERROR,
    DTV_CARD_GENERAL_ERROR,
    DTV_CARD_NOTINITIALIZED
};

struct CDTVStatus
{
    // set by directshow core
    DWORD SmartCardStatus;  // Smart card status
    double CN;		    // C/N rate
    DWORD BitRate;	    // Stream bitrate. Divide by (1 << 17) to get Mbit/sec.
    DWORD CCErrorCount;	    // Continuity error counter.

    BOOL Recording;	    // Is app recording
    time_t RecordTime;	    // total record time, in seconds
    time_t RecordRemainTime;// remaining record time, in seconds

    // set by app
    BOOL Preview;	    // Is preview visible
};

struct CDTVProgram
{
    WCHAR service_name[32]; // unicode program name
    int program_number;	    // program_number aka service_number

    WCHAR event_name[255];  // from EIT p Unicode
    WCHAR event_text[255];  // from EIT p Unicode
    UINT event_start_time;  // from EIT p 24bit BCD ex 08:20:00 = 0x00082000
    UINT event_duration;    // from EIT p 24bit BCD ex 00:25:00	= 0x00002500

    USHORT vpid;	    // video PID
    USHORT apid[4];	    // audio PIDs {up to 4}
    USHORT spid;	    // subtitle PID
    BYTE videotype;	    // type of video
    BYTE audiotype;	    // type of audio
    LONG mask;		    // some flags
};

class IDTVApp
{
public:
    virtual ~IDTVApp() { }

    // change channel
    virtual BOOL WINAPI put_Channel(int Channel) =0;
    // get current channel
    virtual int WINAPI get_Channel(void) =0;
    // set service ID
    virtual BOOL WINAPI put_ServiceID(int ProgramNumber) =0;
    // get current service ID
    virtual int WINAPI get_ServiceID() =0;
    // Get a list of current programs and some data about them
    // @Count [in, out] = number of items in Programs array
    // Returns false if Programs is not large enough, and sets Count to actual number
    virtual BOOL WINAPI GetProgramList(CDTVProgram *Programs, int *Count) =0;
    // signal the application to exit
    virtual void WINAPI TerminateApp(void) =0;

    // ---------- VERSION 1.4 ADD
    // get b-cas information, signal ratio, bitrate, cc error counts, recording information, preview information.
    virtual BOOL WINAPI GetStatus(CDTVStatus *Status) =0;
    // convert arib-encoded text data into UNICODE
    virtual BOOL WINAPI ParseISO2022Text(BYTE *SrcBuf, int SrcLen, WCHAR *DestBuf, int DestLen) =0;
};

class IDTVPlugin
{
    protected:
        virtual ~IDTVPlugin() { }

    public:
	// Host calls this function first to set Host API pointer and SDK version. If it fails, plugin is skipped.
	virtual BOOL WINAPI SetHostAPI(IDTVApp *Application, DWORD Version) =0;
	// Host calls this function to initialize plugin. If something goes wrong, return FALSE and plugin will be skipped.
	virtual BOOL WINAPI Init(HWND AppWindow) =0;
	// Host calls this function to pass a buffer of 188-byte aligned TS data
	virtual BOOL WINAPI ProcessTS(BYTE *Buffer, DWORD Length) =0;
	// Host calls this function on channel or service change.
	virtual BOOL WINAPI OnStreamChange(int Channel, int ServiceID) =0;
	// ------------ VERSION 1.4 ADD
	// Host calls this function before saving a file with a requested filename.
	virtual BOOL WINAPI OnFileSave(TCHAR *Buffer, DWORD Length) =0;
};

typedef IDTVPlugin *(*GetIfacePtr)(void);	    // deprecated
typedef BOOL (WINAPI *GetIfacePtr2)(DWORD dwSDKVersion, void **ppvObject);
